--
-- wetThreshingDamage
-- 
-- @idea: 		Druptown (www.fs-uk.com)
-- @author:    	Xentro (www.fs-uk.com)(Marcus@Xentro.se)
-- @version:    v1.0
-- @date:       2013-03-10
-- 
--
wetThreshingDamage = {};
wetThreshingDamage.modDir = g_currentModDirectory;
wetThreshingDamage.isLoaded = false;

addModEventListener(wetThreshingDamage);

function wetThreshingDamage:loadMap(name)
	if not wetThreshingDamage.isLoaded then
		self.wetThreshingDamageOverlay = Overlay:new("wetThreshingDamageOverlay", (wetThreshingDamage.modDir .. "fieldswet.png"), 0.925, 0.55, 0.055, 0.073333333333333);
		
		self.currentDamage = 0;
		self.dryTime = 240; -- 4 hours
		self.damageTime = 30; -- 30 minutes
		
		self.rainTypes = {};
		self:addRainType("rain");
		self:addRainType("hail");
		
		self.exceptionFruits = {};
		self:addExceptionFruit(FruitUtil.FRUITTYPE_SUGARBEET);
		self:addExceptionFruit(FruitUtil.FRUITTYPE_POTATO);
		self:addExceptionFruit(FruitUtil.FRUITTYPE_MAIZE);
		self:addExceptionFruit(FruitUtil.FRUITTYPE_CHAFF);
		self:addExceptionFruit(FruitUtil.FRUITTYPE_GRASS);
		
		self.lastRainTime = 0;
		self.damageBeforeRain = 0;
		self.savedLastRainScale = 0;
		
		self.startRealDamage = 0.18;
		self.AIStopPercentage = self.startRealDamage;
		
		g_currentMission.rainDamage = self;
		
		if g_currentMission.missionInfo.isValid then
			self:loadFromAttributes();
		end;
		wetThreshingDamage.isLoaded = true;
	end;
end;

function wetThreshingDamage:deleteMap()
	if self.wetThreshingDamageOverlay ~= nil then
		self.wetThreshingDamageOverlay:delete();
	end;
	
	for rain in ipairs(self.rainTypes) do
		self:removeRainType(rain);
	end;
	
	for fruit in ipairs(self.exceptionFruits) do
		self:removeExceptionFruit(fruit);
	end;
	
	g_currentMission.rainDamage = nil;
	wetThreshingDamage.isLoaded = false;
end;

function wetThreshingDamage:loadFromAttributes()	
	local savegamePath = g_currentMission.missionInfo.savegameDirectory .. "/wetThreshingDamage.xml";
	local xmlFile = loadXMLFile("tempWetThreshingDamage", savegamePath);
	local key = "wetThreshingDamage";
	
	self.lastRainType = getXMLString(xmlFile, key .. "#lastRainType");
	
	local damageBeforeRain = getXMLFloat(xmlFile, key .. "#damageBeforeRain");
	if damageBeforeRain ~= nil then
		self.damageBeforeRain = damageBeforeRain;
	end;
	
	local lastRainTime = getXMLFloat(xmlFile, key .. "#lastRainTime");
	if lastRainTime ~= nil then
		self.lastRainTime = lastRainTime;
	end;
	
	local savedLastRainScale = getXMLFloat(xmlFile, key .. "#savedLastRainScale");
	if savedLastRainScale ~= nil then
		self.savedLastRainScale = savedLastRainScale;
	end;
	
	local currentDamage = getXMLFloat(xmlFile, key .. "#currentDamage");
	if currentDamage ~= nil then
		self.currentDamage = currentDamage;
	end;
	
	delete(xmlFile);
end;

function wetThreshingDamage:getSaveAttributes()
	local savegame = self.savegames[self.selectedIndex];
	if savegame ~= nil then
		local key = "wetThreshingDamage";
		local xmlFile = createXMLFile("tempWetThreshingDamage", savegame.savegameDirectory .. "/wetThreshingDamage.xml", key);
		local wet = g_currentMission.rainDamage;
		
		if wet.lastRainType ~= nil then
			setXMLString(xmlFile, key .. "#lastRainType", wet.lastRainType);
		end;
		setXMLFloat(xmlFile, key .. "#damageBeforeRain", wet.damageBeforeRain);
		setXMLFloat(xmlFile, key .. "#lastRainTime", wet.lastRainTime);
		setXMLFloat(xmlFile, key .. "#savedLastRainScale", wet.savedLastRainScale);
		setXMLFloat(xmlFile, key .. "#currentDamage", wet.currentDamage);
		
		saveXMLFile(xmlFile);
		delete(xmlFile);
	end;
end;
CareerScreen.saveSelectedGame = Utils.appendedFunction(CareerScreen.saveSelectedGame, wetThreshingDamage.getSaveAttributes);

function wetThreshingDamage:mouseEvent(posX, posY, isDown, isUp, button)
end;

function wetThreshingDamage:keyEvent(unicode, sym, modifier, isDown)
end;

function wetThreshingDamage:update(dt)
	local g = g_currentMission.environment;
	
	if g.currentRain ~= nil and self:checkRain(g.currentRain.rainTypeId) and g.lastRainScale > 0.01 and self.lastRainType == nil then
		self.lastRainType = g.currentRain.rainTypeId;
		
	elseif g.timeSinceLastRain > self.dryTime and self.lastRainType ~= nil then
		self.lastRainType = nil;
		self.damageBeforeRain = 0;
		
	elseif g.timeSinceLastRain > 0 and self.lastRainType ~= nil then
		self.lastRainTime = 0;
		self.savedLastRainScale = 0;
		self.damageBeforeRain = self.currentDamage;
	end;
	
	if self.lastRainType ~= nil then
		if g.lastRainScale > 0.01 then
			local dtMinutes = dt / 60000 * g.timeScale;
			
			self.lastRainTime = self.lastRainTime + dtMinutes;
			
			if g.lastRainScale > self.savedLastRainScale then
				self.savedLastRainScale = g.lastRainScale;
			end;
			
			local damageTime = (self.lastRainTime / 2) * self.savedLastRainScale;
			self.currentDamage = math.min((damageTime / self.damageTime) + self.damageBeforeRain, 0.5);
		end;
		
		if g.timeSinceLastRain > 0 then
			local dryTime = self.dryTime - g.timeSinceLastRain;
			self.currentDamage = math.max((dryTime / self.dryTime) / 2, 0);
		end;
	end;
end;

function wetThreshingDamage:draw()
end;

function wetThreshingDamage:addRainType(rain)
	self.rainTypes[rain] = true;
end;

function wetThreshingDamage:removeRainType(rain)
	self.rainTypes[rain] = nil;
end;

function wetThreshingDamage:checkRain(rainType)
	if self.rainTypes[rainType] ~= nil then
		return true;
	end;
	return false;
end;

function wetThreshingDamage:addExceptionFruit(fruit)
	self.exceptionFruits[fruit] = true;
end;

function wetThreshingDamage:removeExceptionFruit(fruit)
	self.exceptionFruits[fruit] = nil;
end;

function wetThreshingDamage:checkFruit(fruit, vehicle)
	if self.exceptionFruits[fruit] ~= nil and self.exceptionFruits[fruit] then
		return true;
	end;
	return false;
end;



wetThreshingCombine = {};

function wetThreshingCombine:load(xmlFile)
	self.allowThreshingDuringRain = true;
	
	self.rainDamage = {};
	self.rainDamage.activated = false;
	self.rainDamage.override = false;
end;
Combine.load = Utils.appendedFunction(Combine.load, wetThreshingCombine.load);

function wetThreshingCombine:updateTick(dt)
	local curFillType = self.lastValidInputFruitType;
	if curFillType ~= nil and self.rainDamage ~= nil and not self.rainDamage.override and g_currentMission.rainDamage ~= nil then		
		if g_currentMission.rainDamage.checkFruit ~= nil and not g_currentMission.rainDamage:checkFruit(curFillType) then
			if not self.rainDamage.activated then
				self.rainDamage.activated = true;
			end;
		else
			if self.rainDamage.activated then
				self.rainDamage.activated = false;
			end;
		end;
	end;
end;
Combine.updateTick = Utils.appendedFunction(Combine.updateTick, wetThreshingCombine.updateTick);

function wetThreshingCombine:draw()
	if g_currentMission.rainDamage ~= nil and g_currentMission.rainDamage.lastRainType ~= nil and self.rainDamage ~= nil and self.rainDamage.activated then
		if g_currentMission.rainDamage.wetThreshingDamageOverlay ~= nil then
			g_currentMission.rainDamage.wetThreshingDamageOverlay:render();
			setTextBold(true);
			if g_currentMission.rainDamage.currentDamage >= g_currentMission.rainDamage.startRealDamage then
				setTextColor(1, 0, 0, 1);
			else
				setTextColor(1, 1, 1, 1);
			end;
			setTextAlignment(RenderText.ALIGN_CENTER);
			renderText(g_currentMission.rainDamage.wetThreshingDamageOverlay.x, g_currentMission.rainDamage.wetThreshingDamageOverlay.y - 0.035, 0.018, string.format(g_i18n:getText("Yield_damage"), (100 * g_currentMission.rainDamage.currentDamage)));
			setTextAlignment(RenderText.ALIGN_LEFT);
			setTextBold(false);
		end;
	end;
end;
Combine.draw = Utils.appendedFunction(Combine.draw, wetThreshingCombine.draw);

wetThreshingCombine.setGrainTankFillLevel = function(self, fillLevel, fruitType)
	if g_currentMission.rainDamage ~= nil and g_currentMission.rainDamage.lastRainType ~= nil and self.rainDamage ~= nil and self.rainDamage.activated then
		if g_currentMission.rainDamage.currentDamage >= g_currentMission.rainDamage.startRealDamage then
			local deltaFillLevel = fillLevel - self.grainTankFillLevel;
			deltaFillLevel = deltaFillLevel - (deltaFillLevel * g_currentMission.rainDamage.currentDamage);
			fillLevel = self.grainTankFillLevel + deltaFillLevel;
		end;
	end;
end;
Combine.setGrainTankFillLevel = Utils.prependedFunction(Combine.setGrainTankFillLevel, wetThreshingCombine.setGrainTankFillLevel);

wetThreshingCombine.getIsAIThreshingAllowed = function(self, superFunc)
	if g_currentMission.rainDamage ~= nil and g_currentMission.rainDamage.lastRainType ~= nil and self.rainDamage ~= nil and self.rainDamage.activated then
		if g_currentMission.rainDamage.currentDamage >= g_currentMission.rainDamage.AIStopPercentage then
			return false;
		end;
	end;
	if superFunc ~= nil then
		return superFunc(self);
	end;
	return true;
end;
AICombine.getIsAIThreshingAllowed = Utils.overwrittenFunction(AICombine.getIsAIThreshingAllowed, wetThreshingCombine.getIsAIThreshingAllowed);